"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$average = exports.$max = exports.$min = exports.$sum = exports.$last = exports.$first = exports.$map = exports.$mapset = void 0;
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const types_1 = require("./types");
function $mapset(values, callback) {
    if (!(0, commons_1.$ok)(callback)) {
        callback = function (v, _) { return v; };
    }
    const ret = new Set();
    if ((0, commons_1.$ok)(values)) {
        let index = 0;
        for (let v of values) {
            const mv = callback(v, index);
            if ((0, commons_1.$ok)(mv)) {
                ret.add(mv);
            }
            index++;
        }
    }
    return ret;
}
exports.$mapset = $mapset;
function $map(values, callback) {
    const ret = new Array();
    if ((0, commons_1.$ok)(values)) {
        let index = 0;
        for (let v of values) {
            const mv = callback(v, index);
            if ((0, commons_1.$ok)(mv)) {
                ret.push(mv);
            }
            index++;
        }
    }
    return ret;
}
exports.$map = $map;
function $first(values) {
    const n = (0, commons_1.$count)(values);
    return n > 0 ? values[0] : undefined;
}
exports.$first = $first;
function $last(values) {
    const n = (0, commons_1.$count)(values);
    return n > 0 ? values[n - 1] : undefined;
}
exports.$last = $last;
function $sum(values) {
    const [, , , sum] = _countsAndSum(values);
    return sum;
}
exports.$sum = $sum;
function $min(values) { return _minmax(values, types_1.Descending); }
exports.$min = $min;
function $max(values) { return _minmax(values, types_1.Ascending); }
exports.$max = $max;
function $average(values, opts = {}) {
    let [count, definedCount, okCount, sum] = _countsAndSum(values);
    if (opts.countsOnlyOKItems) {
        count = okCount;
    }
    else if (opts.countsOnlyDefinedItems) {
        count = definedCount;
    }
    return (0, commons_1.$defined)(sum) && count > 0 ? sum / count : undefined;
}
exports.$average = $average;
Array.prototype.average = function average(opts) { return $average(this, opts); };
Array.prototype.filteredMap = function filteredMap(callback) { return $map(this, callback); };
Array.prototype.filteredSet = function filteredMap(callback) { return $mapset(this, callback); };
Array.prototype.first = function first() { return $first(this); };
Array.prototype.fusionEnumeration = function fusionEnumeration() { return this; };
Array.prototype.last = function first() { return $last(this); };
Array.prototype.max = function max() { return $max(this); };
Array.prototype.min = function min() { return $min(this); };
Array.prototype.sum = function sum() { return $sum(this); };
function _countsAndSum(values) {
    let sum = 0;
    let validCount = 0;
    let definedCount = 0;
    let totalCount = 0;
    if ((0, commons_1.$ok)(values)) {
        for (let v of values) {
            if ((0, commons_1.$defined)(v)) {
                if (v !== null) {
                    if ((0, commons_1.$defined)(sum)) {
                        let n = undefined;
                        if ((0, commons_1.$isnumber)(v)) {
                            n = v;
                        }
                        else if ((0, commons_1.$isstring)(v)) {
                            n = Number(v);
                        }
                        else if ((0, commons_1.$ismethod)(v, 'toNumber')) {
                            n = v.toNumber();
                        }
                        if (!(0, commons_1.$isnumber)(n)) {
                            sum = undefined;
                        }
                        else {
                            sum += n;
                        }
                    }
                    validCount++;
                }
                definedCount++;
            }
            totalCount++;
        }
    }
    return [totalCount, definedCount, validCount, sum];
}
function _minmax(values, compValue) {
    let ret = undefined;
    if ((0, commons_1.$ok)(values)) {
        for (let v of values) {
            if (!(0, commons_1.$ok)(v)) {
                return undefined;
            }
            else if (!(0, commons_1.$ok)(ret)) {
                ret = v;
            }
            else {
                let comp = (0, compare_1.$compare)(ret, v);
                if (!(0, commons_1.$defined)(comp)) {
                    return undefined;
                }
                else if (comp === compValue) {
                    ret = v;
                }
            }
        }
    }
    return ret;
}
//# sourceMappingURL=array.js.map