/// <reference types="node" />
import { Nullable, StringEncoding, TSDataLike, uint, uint16, uint32, UUID, UUIDVersion } from './types';
import { TSCharset } from './tscharset';
import { TSData } from './tsdata';
export type HashMethod = 'SHA224' | 'SHA256' | 'SHA384' | 'SHA512' | 'MD5' | 'SHA1';
export declare const SHA224: HashMethod;
export declare const SHA256: HashMethod;
export declare const SHA384: HashMethod;
export declare const SHA512: HashMethod;
export declare const MD5: HashMethod;
export declare const SHA1: HashMethod;
export type EncryptionAlgorithm = 'AES128' | 'AES256';
export declare const AES128: EncryptionAlgorithm;
export declare const AES256: EncryptionAlgorithm;
export declare function $uuid(internalImplementation?: boolean): UUID;
export declare function $slowuuid(convertToLowerCase?: boolean): UUID;
export declare function $uuidVersion(str: Nullable<string>): UUIDVersion | undefined;
export declare function $setCommonItializationVector(d?: Nullable<TSDataLike>): void;
export declare function $commonInitializationVectorCopy(): Buffer;
export interface $encryptOptions {
    encoding?: Nullable<StringEncoding | TSCharset>;
    keyEncoding?: Nullable<StringEncoding | TSCharset>;
    algorithm?: Nullable<EncryptionAlgorithm>;
    noInitializationVector?: Nullable<boolean>;
    dataOutput?: Nullable<boolean>;
}
export declare function $encrypt(src: string | TSDataLike, skey: string | TSDataLike, opts?: Nullable<$encryptOptions>): TSData | string | null;
export interface $decryptOptions extends $encryptOptions {
}
export declare function $decrypt(source: string | TSDataLike, skey: string | TSDataLike, opts?: Nullable<$decryptOptions>): TSData | string | null;
export declare function $crc16(source: string | TSDataLike, encoding?: Nullable<StringEncoding | TSCharset>): uint16;
export declare function $crc32(source: string | TSDataLike, encoding?: Nullable<StringEncoding | TSCharset>): uint32;
export declare function $hash(buf: string | TSDataLike, method?: Nullable<HashMethod>, encoding?: Nullable<StringEncoding | TSCharset>): string | null;
export interface $hashOptions {
    encoding?: Nullable<StringEncoding | TSCharset>;
    method?: Nullable<HashMethod>;
    separator?: Nullable<string>;
    dataOutput?: Nullable<boolean>;
}
export declare function $slowhash(source: string | TSDataLike, options?: $hashOptions): string | Buffer;
export declare function $hashfile(filePath: Nullable<string>, method?: Nullable<HashMethod>): Promise<string | null>;
export declare function $uuidhash(buf: string | TSDataLike, version?: Nullable<UUIDVersion>): string | null;
export declare function $uuidhashfile(filePath: Nullable<string>, version?: Nullable<UUIDVersion>): Promise<string | null>;
export declare function $random(max?: Nullable<number>): uint;
export interface $passwordOptions {
    hasLowercase?: boolean;
    hasUppercase?: boolean;
    hasNumeric?: boolean;
    hasSpecials?: boolean;
}
export declare function $password(len: number, opts?: $passwordOptions): string | null;
declare global {
    export interface String {
        crc16: (this: string, encoding?: Nullable<StringEncoding | TSCharset>) => uint16;
        crc32: (this: string, encoding?: Nullable<StringEncoding | TSCharset>) => uint32;
        hash: (this: string, method?: Nullable<HashMethod>, encoding?: Nullable<StringEncoding | TSCharset>) => string | null;
        slowhash: (this: string, options?: $hashOptions) => string | Buffer;
    }
    export interface Uint8Array {
        crc16: (this: any) => uint16;
        crc32: (this: any) => uint32;
        hash: (this: any, method?: Nullable<HashMethod>) => string | null;
        slowhash: (this: any, options?: $hashOptions) => string | Buffer;
        uuidhash: (this: any, version?: Nullable<UUIDVersion>) => string | null;
    }
    export interface ArrayBuffer {
        crc16: (this: any) => uint16;
        crc32: (this: any) => uint32;
        hash: (this: any, method?: Nullable<HashMethod>) => string | null;
        slowhash: (this: any, options?: $hashOptions) => string | Buffer;
        uuidhash: (this: any, version?: Nullable<UUIDVersion>) => string | null;
    }
}
export declare function $md5(buf: Nullable<TSDataLike>, separator?: Nullable<string>, dataoutput?: Nullable<boolean>): string | Buffer;
export declare function $sha1(buf: Nullable<TSDataLike>, separator?: Nullable<string>, dataoutput?: Nullable<boolean>): string | Buffer;
export declare function $sha224(buf: Nullable<TSDataLike>, separator?: Nullable<string>, dataoutput?: Nullable<boolean>): string | Buffer;
export declare function $sha256(buf: Nullable<TSDataLike>, separator?: Nullable<string>, dataoutput?: Nullable<boolean>): string | Buffer;
export declare function $sha384(buf: Nullable<TSDataLike>, separator?: Nullable<string>, dataoutput?: Nullable<boolean>): string | Buffer;
export declare function $sha512(buf: Nullable<TSDataLike>, separator?: Nullable<string>, dataoutput?: Nullable<boolean>): string | Buffer;
export declare function _insideHash512(H: number[], W: number[]): (M: number[]) => void;
