/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
import { Stats, StatOptions } from 'fs';
import { TSData } from './tsdata';
import { Nullable, StringEncoding, TSDataLike } from './types';
import { TSCharset } from './tscharset';
export declare function $stats(src: Nullable<string>): Nullable<Stats>;
export interface CommonStats {
    accessedAt: number | bigint;
    changedAt: number | bigint;
    createdAt: number | bigint;
    directory: boolean;
    executable: boolean;
    exists: boolean;
    file: boolean;
    gid: number | bigint;
    mode: number | bigint;
    modifiedAt: number | bigint;
    readable: boolean;
    size: number | bigint;
    uid: number | bigint;
    writable: boolean;
}
export declare function $commonstats(src: Nullable<string>, opts?: Nullable<StatOptions>): CommonStats;
export declare function $isfile(src: Nullable<string>): boolean;
export declare function $isexecutable(src: Nullable<string>): boolean;
export declare function $isexecutablefile(src: Nullable<string>): boolean;
export declare function $isreadable(src: Nullable<string>): boolean;
export declare function $chmod(src: Nullable<string>, mode: number): boolean;
export declare function $iswritable(src: Nullable<string>): boolean;
export declare function $isdirectory(src: Nullable<string>): boolean;
export declare function $createDirectory(p: Nullable<string>): boolean;
export declare function $filesize(src: Nullable<string>): number;
export declare function $temporarypath(ext?: Nullable<string>, src?: Nullable<string>): string;
export declare function $uniquefile(src?: Nullable<string>, ext?: Nullable<string>, internalImplementation?: boolean): string;
export declare function $isabsolutepath(src?: Nullable<string>, internalImplementation?: boolean): boolean;
export { $isabsolutepath as $isabsolute };
export declare function $normalizepath(src?: Nullable<string>, internalImplementation?: boolean): string;
export declare function $fromposix(src?: Nullable<string>): string;
export declare function $absolute(src: Nullable<string>): string;
export declare function $homedirectory(): string;
export declare function $currentdirectory(): string;
export declare function $path(first: string | boolean, ...paths: string[]): string;
export declare function $ext(s: Nullable<string>, internalImplementation?: boolean): string;
export declare function $withoutext(s: Nullable<string>, internalImplementation?: boolean): string;
export declare function $newext(s: Nullable<string>, e?: Nullable<string>, internalImplementation?: boolean): string;
export declare function $dir(source: Nullable<string>, internalImplementation?: boolean): string;
export declare function $filename(s: Nullable<string>, internalImplementation?: boolean): string;
export declare function $loadJSON(source: Nullable<string | TSDataLike>): any;
export declare function $readString(src: Nullable<string>, encoding?: Nullable<StringEncoding | TSCharset>): string | null;
export interface BasicWriteOptions {
    attomically?: Nullable<boolean>;
    removePrecedentVersion?: Nullable<boolean>;
    mode?: Nullable<number>;
}
export interface $writeStringOptions extends BasicWriteOptions {
    encoding?: Nullable<StringEncoding | TSCharset>;
    stringStart?: Nullable<number>;
    stringEnd?: Nullable<number>;
}
export declare function $writeString(src: Nullable<string>, str: string, opts?: $writeStringOptions): boolean;
export declare function $fullWriteString(src: Nullable<string>, str: string, opts: $writeStringOptions): [boolean, string | null];
export declare function $readBuffer(src: Nullable<string>): Buffer | null;
export declare function $readData(src: Nullable<string>): TSData | null;
export interface $writeBufferOptions extends BasicWriteOptions {
    byteStart?: number;
    byteEnd?: number;
}
export declare function $writeBuffer(src: Nullable<string>, buf: TSData | NodeJS.ArrayBufferView, opts?: $writeBufferOptions): boolean;
export declare function $fullWriteBuffer(src: Nullable<string>, buf: TSData | NodeJS.ArrayBufferView, opts: $writeBufferOptions): [boolean, string | null];
export declare function $localRenameFile(src: Nullable<string>, dest: Nullable<string>): boolean;
export declare function $removeFile(src: Nullable<string>): boolean;
export declare function $realMoveFile(src: Nullable<string>, dest: Nullable<string>): boolean;
export declare function $copyFile(src: Nullable<string>, dest: Nullable<string>, overwrite?: boolean): boolean;
