"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$meters = exports.$octets = exports.$unit = exports.$fpad = exports.$fpad4 = exports.$fpad3 = exports.$fpad2 = exports.$round = exports.$icast = exports.$div = void 0;
const commons_1 = require("./commons");
const strings_1 = require("./strings");
const string_tables_1 = require("./string_tables");
const tsdate_1 = require("./tsdate");
const types_1 = require("./types");
function $div(a, b) { return $icast(a / b); }
exports.$div = $div;
function $icast(v) { return v >= 0 ? (v <= types_1.UINT32_MAX ? v | 0 : Math.floor(v)) : (v >= types_1.INT32_MIN ? -((-v) | 0) : -Math.floor(-v)); }
exports.$icast = $icast;
function $round(v, decimalPlaces = 0) {
    var p = Math.pow(10, (0, commons_1.$toint)(decimalPlaces));
    return Math.round((v * p) * (1 + Number.EPSILON)) / p;
}
exports.$round = $round;
function $fpad2(v, failedChar) { return $fpad(v, 2, failedChar); }
exports.$fpad2 = $fpad2;
function $fpad3(v, failedChar) { return $fpad(v, 3, failedChar); }
exports.$fpad3 = $fpad3;
function $fpad4(v, failedChar) { return $fpad(v, 4, failedChar); }
exports.$fpad4 = $fpad4;
function $fpad(v, pad, failedChar) {
    const isUnsigned = (0, commons_1.$isunsigned)(v);
    return (isUnsigned ? v.toString() : '').padStart(pad, isUnsigned ? '0' : (0, commons_1.$length)(failedChar) === 1 ? failedChar : 'X');
}
exports.$fpad = $fpad;
const TSUnitMultiples = ['y', 'z', 'a', 'f', 'p', 'n', 'µ', 'm', '', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'];
const TSLog1000 = Math.log(1000);
function $unit(n, opts = {}) {
    const v = (0, commons_1.$ok)(n) ? n : 0;
    const sn = (0, strings_1.$ftrim)(opts.unitName);
    const su = (0, strings_1.$ftrim)(opts.unit);
    const unitName = sn.length ? sn : (su.length ? su : 'm');
    const minU = (0, commons_1.$ok)(opts.minimalUnit) ? Math.min(0, Math.max(-8, opts.minimalUnit)) : -8;
    const maxU = (0, commons_1.$ok)(opts.maximalUnit) ? Math.min(8, Math.max(0, opts.maximalUnit)) : 8;
    let dm = (0, commons_1.$isunsigned)(opts.decimalPlaces) ? opts.decimalPlaces : 2;
    if (v === 0) {
        if (dm === 0 || opts.ignoreZeroDecimals || (minU === 0 && opts.ignoreMinimalUnitDecimals)) {
            return '0 ' + unitName;
        }
        return '0.'.padEnd(2 + dm, '0') + ' ' + unitName;
    }
    const unit = su.length ? su : unitName.charAt(0);
    const i = Math.max(minU, Math.min(maxU, Math.floor(Math.log(Math.abs(v)) / TSLog1000)));
    if (i === minU && opts.ignoreMinimalUnitDecimals) {
        dm = 0;
    }
    return ((0.0 + v) / (0.0 + Math.pow(1000, i))).toFixed(dm) + ' ' + TSUnitMultiples[i + 8] + (i == 0 ? unitName : unit);
}
exports.$unit = $unit;
function $octets(n, decimalPlaces = 2) {
    return $unit(n, {
        decimalPlaces: decimalPlaces,
        unit: 'o',
        unitName: 'octets',
        minimalUnit: 0,
        ignoreZeroDecimals: true,
        ignoreMinimalUnitDecimals: true
    });
}
exports.$octets = $octets;
function $meters(n, decimalPlaces = 2) {
    return $unit(n, { decimalPlaces: decimalPlaces });
}
exports.$meters = $meters;
const FoundationHexaChars = '0123456789ABCDEF';
const FoundationHexaLowerChars = '0123456789abcdef';
Number.prototype.fpad = function fpad(pad, failedChar) { return $fpad(this, pad, failedChar); };
Number.prototype.fpad2 = function fpad(failedChar) { return $fpad(this, 2, failedChar); };
Number.prototype.fpad3 = function fpad(failedChar) { return $fpad(this, 3, failedChar); };
Number.prototype.fpad4 = function fpad(failedChar) { return $fpad(this, 4, failedChar); };
Number.prototype.hexaValue = function hexaValue() {
    if (this >= 48 && this < 58) {
        return this - 48;
    }
    if (this >= 65 && this < 71) {
        return this - 55;
    }
    if (this >= 97 && this < 103) {
        return this - 87;
    }
    return -1;
};
Number.prototype.icast = function iCast() { return $icast(this); };
Number.prototype.isNewLine = function isNewLine() { return string_tables_1.FoundationNewLineNumberCodeSet.has(this); };
Number.prototype.isWhiteSpace = function isWhiteSpace() { return string_tables_1.FoundationWhiteSpacesNumberCodeSet.has(this); };
Number.prototype.isStrictWhiteSpace = function isWhiteSpace() { return string_tables_1.FoundationStricWhiteSpacesNumberCodeSet.has(this); };
Number.prototype.meters = function meters(decimalPlaces) { return $meters(this, decimalPlaces); };
Number.prototype.octets = function octets(decimalPlaces) { return $octets(this, decimalPlaces); };
Number.prototype.round = function round(decimalPlaces) { return $round(this, decimalPlaces); };
Number.prototype.singular = function singular() { return this === 1; };
Number.prototype.toDate = function toDate() { return (0, commons_1.$isnumber)(this) ? new Date(this) : null; };
Number.prototype.toHex1 = function toHex1(toLowerCase) {
    const r = !!toLowerCase ? FoundationHexaLowerChars : FoundationHexaChars;
    return r[this.toUnsigned() & 0x0F];
};
Number.prototype.toHex2 = function toHex2(toLowerCase) {
    const n = this.toUnsigned();
    const r = !!toLowerCase ? FoundationHexaLowerChars : FoundationHexaChars;
    return r[(n >> 4) & 0xF] + r[n & 0xF];
};
Number.prototype.toHex4 = function toHex4(toLowerCase) {
    const n = this.toUnsigned();
    const r = !!toLowerCase ? FoundationHexaLowerChars : FoundationHexaChars;
    return r[(n >> 12) & 0xF] + r[(n >> 8) & 0xF] + r[(n >> 4) & 0xF] + r[n & 0xF];
};
Number.prototype.toHex8 = function toHex8(toLowerCase) {
    const n = this.toUnsigned();
    const r = !!toLowerCase ? FoundationHexaLowerChars : FoundationHexaChars;
    return r[(n >> 28) & 0xF] + r[(n >> 24) & 0xF] + r[(n >> 20) & 0xF] + r[(n >> 16) & 0xF] + r[(n >> 12) & 0xF] + r[(n >> 8) & 0xF] + r[(n >> 4) & 0xF] + r[n & 0xF];
};
Number.prototype.toInt = function toInt(defaultValue) { return (0, commons_1.$toint)(this, defaultValue); };
Number.prototype.toTSDate = function toTSDate() { return tsdate_1.TSDate.fromTimeStamp(this); };
Number.prototype.toUnsigned = function toUnsigned(defaultValue) { return (0, commons_1.$tounsigned)(this, defaultValue); };
Number.prototype.unit = function unit(opts) { return $unit(this, opts); };
//# sourceMappingURL=number.js.map