/// <reference types="node" />
import { TSData } from './tsdata';
import { Bytes, Nullable, StringEncoding, TSDataLike, uint8 } from './types';
export declare function $charset(value: Nullable<StringEncoding | TSCharset>, defaultCharset?: TSCharset): TSCharset;
export declare abstract class TSCharset {
    private static __charsetsMap;
    private static __cachedCharsets;
    static asciiCharset(): TSCharset;
    static latin1Charset(): TSCharset;
    static binaryCharset(): TSCharset;
    static utf8Charset(): TSCharset;
    static unicodeCharset(): TSCharset;
    static ansiCharset(): TSCharset;
    static macCharset(): TSCharset;
    readonly name: string;
    readonly aliases: string[];
    protected constructor(name: string, aliases?: string[]);
    static systemCharset(): TSCharset;
    static encoding(enc: StringEncoding): TSCharset;
    static charset(name: string): TSCharset | undefined;
    private static _cachedCharset;
    abstract stringFromBytes(source: Bytes, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): string;
    abstract stringToBytes(source: string, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): Bytes;
    stringFromData(source: TSDataLike, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): string;
    bufferFromString(source: string, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): Buffer;
    uint8ArrayFromString(source: string, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): Uint8Array;
    bytesFromString(source: string, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): uint8[];
    dataFromString(source: string, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
}
export declare class TSSystemCharset extends TSCharset {
    stringToBytes(source: string, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): Bytes;
    private _stringFromBuffer;
    stringFromBytes(source: Bytes, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): string;
    stringFromData(src: TSDataLike, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): string;
}
