/// <reference types="node" />
import { $hashOptions, HashMethod } from "./crypto";
import { $writeBufferOptions } from "./fs";
import { TSCharset } from "./tscharset";
import { TSClone, TSLeafInspect, TSObject } from "./tsobject";
import { Bytes, Comparison, Nullable, StringEncoding, TSDataLike, uint16, uint32, uint8, UUIDVersion } from "./types";
declare const customInspectSymbol: unique symbol;
export interface TSDataOptions {
    dontCopySourceBuffer?: boolean;
    fillWithZeros?: boolean;
    allocMethod?: (n: number) => Buffer;
}
export declare class TSData implements Iterable<number>, TSObject, TSLeafInspect, TSClone<TSData> {
    protected _len: number;
    protected _buf: Buffer;
    private _allocFn;
    constructor(source?: Nullable<number | TSDataLike>, opts?: TSDataOptions);
    static fromFile(src: Nullable<string>): TSData | null;
    static fromString(source: Nullable<string>, encoding?: Nullable<StringEncoding | TSCharset>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    leafInspect(): string;
    [customInspectSymbol](depth: number, inspectOptions: any, inspect: any): string;
    [Symbol.iterator](): {
        next: () => {
            done: boolean;
            value: number;
        };
    };
    splice(targetStart: number, deleteCount: number, source?: Nullable<TSDataLike>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>, paddingByte?: Nullable<number>): TSData;
    appendByte(source: uint8): TSData;
    appendBytes(source: Nullable<Bytes>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    appendData(source: Nullable<TSDataLike>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    appendString(source: Nullable<string>, encoding?: Nullable<StringEncoding | TSCharset>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    replaceBytes(source: Nullable<Bytes>, targetStart?: Nullable<number>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    replaceData(source: Nullable<TSDataLike>, targetStart?: Nullable<number>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    replaceString(source: Nullable<string>, targetStart?: Nullable<number>, encoding?: Nullable<StringEncoding | TSCharset>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    writeBigInt64BE(value: bigint, offset?: number): TSData;
    writeBigInt64LE(value: bigint, offset?: number): TSData;
    writeBigUInt64BE(value: bigint, offset?: number): TSData;
    writeBigUInt64LE(value: bigint, offset?: number): TSData;
    writeUInt8(value: number, offset?: number): TSData;
    writeUInt16LE(value: number, offset?: number): TSData;
    writeUInt16BE(value: number, offset?: number): TSData;
    writeUInt32LE(value: number, offset?: number): TSData;
    writeUInt32BE(value: number, offset?: number): TSData;
    writeInt8(value: number, offset?: number): TSData;
    writeInt16LE(value: number, offset?: number): TSData;
    writeInt16BE(value: number, offset?: number): TSData;
    writeInt32LE(value: number, offset?: number): TSData;
    writeInt32BE(value: number, offset?: number): TSData;
    writeFloatLE(value: number, offset?: number): TSData;
    writeFloatBE(value: number, offset?: number): TSData;
    writeDoubleLE(value: number, offset?: number): TSData;
    writeDoubleBE(value: number, offset?: number): TSData;
    removeTraillingNewLines(): TSData;
    removeTraillingSpaces(): TSData;
    removeTraillingStrictSpaces(): TSData;
    removeTraillingZeros(): TSData;
    truncateBy(n: number): TSData;
    get mutableBuffer(): Buffer;
    get internalStorage(): [Buffer, number];
    set length(n: number);
    get capacity(): number;
    get length(): number;
    get byteLength(): number;
    clone(): TSData;
    get buffer(): Buffer;
    entries(): IterableIterator<[number, number]>;
    keys(): IterableIterator<number>;
    values(): IterableIterator<number>;
    includes(value: Nullable<TSDataLike | number | string>, encoding?: Nullable<StringEncoding | TSCharset>): boolean;
    startsWith(value: Nullable<TSData | number | string>, encoding?: Nullable<StringEncoding | TSCharset>): boolean;
    endsWith(value: Nullable<TSDataLike | number | string>, encoding?: Nullable<StringEncoding | TSCharset>): boolean;
    indexOf(value: Nullable<TSDataLike | number | string>, byteOffset?: Nullable<number>, encoding?: Nullable<StringEncoding | TSCharset>): number;
    lastIndexOf(value: Nullable<TSDataLike | number | Uint8Array>, byteOffset?: Nullable<number>, encoding?: Nullable<StringEncoding | TSCharset>): number;
    slice(sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): TSData;
    copy(targetBuffer: Buffer | Uint8Array | TSData, targetStart?: Nullable<number>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): number;
    writeToFile(path: string, opts?: $writeBufferOptions): boolean;
    fullWriteToFile(path: string, opts: $writeBufferOptions): [boolean, string | null];
    equals(otherBuffer: Uint8Array): boolean;
    readBigUInt64BE(offset?: number): bigint;
    readBigUInt64LE(offset?: number): bigint;
    readBigInt64BE(offset?: number): bigint;
    readBigInt64LE(offset?: number): bigint;
    readUInt8(offset?: number): number;
    readUInt16LE(offset?: number): number;
    readUInt16BE(offset?: number): number;
    readUInt32LE(offset?: number): number;
    readUInt32BE(offset?: number): number;
    readInt8(offset?: number): number;
    readInt16LE(offset?: number): number;
    readInt16BE(offset?: number): number;
    readInt32LE(offset?: number): number;
    readInt32BE(offset?: number): number;
    readFloatLE(offset?: number): number;
    readFloatBE(offset?: number): number;
    readDoubleLE(offset?: number): number;
    readDoubleBE(offset?: number): number;
    toBase64(): string;
    crc16(): uint16;
    crc32(): uint32;
    hash(method?: Nullable<HashMethod>): string | null;
    slowhash(options?: $hashOptions): string | Buffer;
    uuidhash(version?: Nullable<UUIDVersion>): string | null;
    toString(encoding?: Nullable<StringEncoding | TSCharset>, sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): string;
    toJSON(): any;
    toBytes(sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): Bytes;
    toArray(sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): number[];
    toArrayBuffer(sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): ArrayBuffer;
    toUint8Array(sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): Uint8Array;
    toBuffer(sourceStart?: Nullable<number>, sourceEnd?: Nullable<number>): Buffer;
    compare(other: any): Comparison;
    compareToData(other: Nullable<TSDataLike>): Comparison;
    isEqual(other: any): boolean;
    isEqualToData(other: Nullable<TSDataLike>): boolean;
    private _splice;
    private _index;
    private _rindex;
    private _insideCopy;
    protected _willGrow(n: number): void;
    protected _read<T>(offset: number | undefined, size: number, bufferReadFn: (offset?: number) => T): T;
    protected _write<T>(value: T, offset: number | undefined, size: number, bufferWriteFn: (value: T, offset?: number) => void): TSData;
}
export interface TSDataConstructor {
    new (source?: Nullable<TSDataLike | number>, opts?: TSDataOptions): TSData;
}
export {};
