import { TSDateComp, TSDateForm } from "./tsdatecomp";
import { IsoDateFormat } from "./commons";
import { Comparison, country, isodate, language, Nullable, uint } from "./types";
import { TSClone, TSLeafInspect, TSObject } from "./tsobject";
import { TSCountry } from "./tscountry";
import { Locales } from "./tsdefaults";
declare const customInspectSymbol: unique symbol;
export declare class TSDate implements TSObject, TSLeafInspect, TSClone<TSDate> {
    private readonly _timestamp;
    static readonly FUTURE = "future";
    static readonly PAST = "past";
    constructor(year: number, month: number, day: number);
    constructor(year: number, month: number, day: number, hours: number, minutes: number, seconds: number);
    constructor(interval: number);
    constructor(date: Date);
    constructor(date: TSDate);
    constructor(stringDate: string);
    constructor(nullDate: null);
    constructor();
    static past(): TSDate;
    static future(): TSDate;
    static zulu(): TSDate;
    static isDateSource(value: any): boolean;
    static fromZulu(s: Nullable<Date>): TSDate | null;
    static fromEpoch(t: Nullable<number>): TSDate | null;
    static from(date: Nullable<number | string | Date>): TSDate | null;
    static fromTimeStamp(t: Nullable<number>): TSDate | null;
    static fromDate(d: Nullable<Date>): TSDate | null;
    static fromComponents(comp: Nullable<TSDateComp>): TSDate | null;
    static fromIsoString(s: Nullable<string>): TSDate | null;
    static fromString(s: Nullable<string>, form?: TSDateForm): TSDate | null;
    static fromDateString(s: Nullable<string>, form?: TSDateForm): TSDate | null;
    clone(): TSDate;
    toTSDate: () => TSDate;
    get timestamp(): number;
    isLeap(): boolean;
    dateWithoutTime(): TSDate;
    timeSinceDate(d: TSDate): number;
    get year(): number;
    get month(): number;
    get day(): number;
    get hour(): number;
    get minute(): number;
    get second(): number;
    week(offset?: number): number;
    dayOfYear(): number;
    dayOfWeek(offset?: number): number;
    lastDayOfMonth(): number;
    dateByAdding(years: number, months?: number, days?: number, hours?: number, minutes?: number, seconds?: number): TSDate;
    dateByAddingWeeks(weeks: number): TSDate;
    dateByAddingDays(days: number): TSDate;
    dateByAddingHours(hours: number): TSDate;
    dateByAddingMinutes(mn: number): TSDate;
    dateByAddingTime(time: number): TSDate;
    firstDateOfYear(): TSDate;
    lastDateOfYear(): TSDate;
    firstDateOfMonth(): TSDate;
    lastDateOfMonth(): TSDate;
    firstDateOfWeek(offset?: number): TSDate;
    daysSinceDate(aDate: TSDate): number;
    toNumber(): number;
    toEpoch(): number;
    toComponents(): TSDateComp;
    toEpochTimestamp(): number;
    toDate(): Date;
    toIsoString(format?: IsoDateFormat): isodate;
    toISOString(): string;
    compare(other: any): Comparison;
    isEqual(other: any): boolean;
    toString(format?: Nullable<TSDateForm | string>, locale?: Nullable<language | country | TSCountry | Locales>): string;
    toLocaleString(format?: Nullable<TSDateForm | string>, locale?: Nullable<language | country | TSCountry | Locales>): string;
    toJSON(): string;
    toArray(): TSDate[];
    leafInspect(): string;
    [customInspectSymbol](depth: number, inspectOptions: any, inspect: any): string;
    private _toString;
}
export declare const TSMinute = 60;
export declare const TSHour = 3600;
export declare const TSDay = 86400;
export declare const TSWeek = 604800;
export declare const TSDaysFrom00000229To20010101 = 730792;
export declare const TSDaysFrom00010101To20010101 = 730485;
export declare const TSSecsFrom00010101To20010101 = 63113904000;
export declare const TSSecsFrom19700101To20010101 = 978307200;
export declare const TSMaxTimeStamp = 8639021692800;
export declare function $isleap(y: number): boolean;
export declare function $dayisvalid(year: number, month: number, day: number): boolean;
export declare function $timeisvalid(hour: number, minute: number, second: number): boolean;
export declare function $timestamp(year: number, month: number, day: number, hours?: number, minutes?: number, seconds?: number): number;
export declare function $secondFromTimestamp(ts: number): uint;
export declare function $minuteFromTimestamp(ts: number): uint;
export declare function $hourFromTimestamp(ts: number): uint;
export declare function $timestampWithoutTime(ts: number): number;
export declare function $dayOfYear(ts: number): number;
export declare function $dayOfWeekFromTimestamp(ts: number, offset?: number): number;
export declare function $weekOfYear(ts: number, offset?: number): number;
export declare function $insettimestamp(ts: number): number;
export {};
