import { country, currency, language, Languages, Nullable, StringDictionary, StringEncoding, StringTranslation } from "./types";
import { TSCountry } from "./tscountry";
import { TSCharset } from "./tscharset";
export interface Locales {
    names: StringTranslation;
    language: language;
    months: string[];
    shortMonths: string[];
    days: string[];
    shortDays: string[];
    startingWeekDay: number;
    dateTimeFormat: string;
    dateFormat: string;
    shortDateFormat: string;
    shortDateTimeFormat: string;
    timeFormat: string;
    partialTimeFormat: string;
    ampm: string[];
}
export interface DefaultsConfigurationOptions {
    encoding?: Nullable<StringEncoding | TSCharset>;
    debug?: Nullable<boolean>;
    underscoreMax?: Nullable<number>;
    variableMax?: Nullable<number>;
}
export type StringTranslations = {
    [key in Languages]?: StringDictionary;
};
export declare class TSDefaults {
    private static __instance;
    private static __locales;
    private _defaultLanguage;
    private _defaultCurrency;
    private _tmpDirectory;
    private _values;
    private _localizations;
    private _countriesMap;
    private _languagesMap;
    private _currenciesMap;
    private _managedLocalesMap;
    private _managedLanguages;
    private constructor();
    managedLanguages(): language[];
    managedLanguage(s?: Nullable<string>): language | null;
    country(s: Nullable<string>): country | null;
    language(s?: Nullable<TSCountry | string>): language | null;
    currency(s?: Nullable<TSCountry | string>): currency | null;
    addLocalizations(lang: language, loc: StringDictionary): void;
    localizations(locale: Nullable<language | country | TSCountry | string>): StringDictionary;
    locales(locale?: Nullable<language | country | TSCountry | string>): Locales;
    get defaultLanguage(): "id" | "br" | "hr" | "li" | "th" | "tr" | "tt" | "so" | "ss" | "ae" | "dz" | "ts" | "aa" | "av" | "ay" | "sq" | "el" | "es" | "ca" | "fr" | "de" | "nl" | "be" | "ru" | "bs" | "sr" | "bg" | "cs" | "da" | "et" | "fi" | "en" | "hu" | "it" | "lv" | "lt" | "lb" | "mk" | "mt" | "ro" | "no" | "pl" | "pt" | "sk" | "sl" | "sv" | "uk" | "ab" | "af" | "ak" | "am" | "an" | "ar" | "as" | "az" | "ba" | "bh" | "bi" | "bm" | "bn" | "bo" | "ce" | "ch" | "co" | "cr" | "cu" | "cv" | "cy" | "dv" | "ee" | "eu" | "fa" | "ff" | "fj" | "fo" | "fy" | "ga" | "gd" | "gl" | "gn" | "gu" | "gv" | "ha" | "he" | "hi" | "ho" | "ht" | "hy" | "hz" | "ig" | "ii" | "ik" | "is" | "iu" | "ja" | "jv" | "ka" | "kg" | "ki" | "kj" | "kk" | "kl" | "km" | "kn" | "ko" | "kr" | "ks" | "ku" | "kv" | "kw" | "ky" | "la" | "lg" | "ln" | "lo" | "lu" | "mg" | "mh" | "mi" | "ml" | "mn" | "mr" | "ms" | "my" | "na" | "nb" | "nd" | "ne" | "ng" | "nn" | "nr" | "nv" | "ny" | "oc" | "oj" | "om" | "or" | "os" | "pa" | "pi" | "ps" | "qu" | "rm" | "rn" | "rw" | "sa" | "sc" | "sd" | "se" | "sg" | "si" | "sm" | "sn" | "st" | "su" | "sw" | "ta" | "te" | "tg" | "ti" | "tk" | "tl" | "tn" | "to" | "tw" | "ty" | "ug" | "ur" | "uz" | "ve" | "vi" | "wa" | "wo" | "xh" | "yi" | "yo" | "za" | "zh" | "zu";
    setDefaultLanguage(l: language): language;
    get defaultCurrency(): "GBP" | "JPY" | "EUR" | "SOS" | "ALL" | "BYN" | "BAM" | "BGN" | "CYP" | "CZK" | "DKK" | "HUF" | "CHF" | "MKD" | "MDL" | "NOK" | "PLN" | "RON" | "RSD" | "SEK" | "AED" | "AFN" | "AMD" | "ANG" | "AOA" | "ARP" | "ARS" | "AUD" | "AWG" | "AZN" | "BBD" | "BDT" | "BHD" | "BIF" | "BMD" | "BND" | "BOB" | "BOV" | "BRL" | "BSD" | "BTN" | "BWP" | "BYR" | "BZD" | "CAD" | "CDF" | "CLF" | "CLP" | "CNY" | "COP" | "COU" | "CRC" | "CUC" | "CUP" | "CVE" | "DJF" | "DOP" | "DZD" | "ECS" | "ECV" | "EGP" | "ERN" | "ETB" | "FJD" | "FKP" | "GEL" | "GHS" | "GIP" | "GMD" | "GNF" | "GTQ" | "GWP" | "GYD" | "HKD" | "HNL" | "HRK" | "HTG" | "IDR" | "ILS" | "INR" | "IQD" | "IRR" | "ISK" | "JMD" | "JOD" | "KES" | "KGS" | "KHR" | "KMF" | "KPW" | "KRW" | "KZT" | "KWD" | "KYD" | "LAK" | "LBP" | "LKR" | "LRD" | "LSL" | "LTL" | "LVL" | "LYD" | "MAD" | "MGA" | "MMK" | "MNT" | "MOP" | "MRU" | "MUR" | "MVR" | "MWK" | "MXN" | "MXV" | "MYR" | "MZN" | "NAD" | "NGN" | "NHF" | "NIO" | "NPR" | "NZD" | "OMR" | "PAB" | "PEN" | "PGK" | "PHP" | "PKR" | "PYG" | "QAR" | "RUB" | "RWF" | "SAR" | "SBD" | "SCR" | "SDG" | "SGD" | "SHP" | "SLL" | "SRD" | "SSP" | "STD" | "SVC" | "SYP" | "SZL" | "THB" | "TJS" | "TMT" | "TND" | "TOP" | "TRY" | "TTD" | "TWD" | "TZS" | "UAH" | "UGX" | "USD" | "USN" | "UYU" | "UYW" | "UZS" | "VEB" | "VES" | "VND" | "VUV" | "WST" | "XAF" | "XAG" | "XAU" | "XCD" | "XDR" | "XFO" | "XFU" | "XOF" | "XPD" | "XPF" | "XPT" | "XSU" | "XUA" | "YER" | "ZAR" | "ZMK" | "ZWL";
    setDefaultCurrency(c: currency): currency;
    get tmpDirectory(): string;
    setTmpDirectory(path: string): void;
    setValue(key: string, value: any): void;
    getValue(key: string): any;
    configure(path?: Nullable<string>, opts?: Nullable<DefaultsConfigurationOptions>): void;
    static defaults(): TSDefaults;
}
export declare function $tmp(): string;
export declare function $locales(locale?: Nullable<language | country | TSCountry>): Locales;
export declare function $country(s: Nullable<string>): country | null;
export declare function $language(s?: Nullable<TSCountry | string>): language | null;
export declare function $currency(s?: Nullable<TSCountry | string>): currency | null;
export declare function $config(path?: Nullable<string>, opts?: Nullable<DefaultsConfigurationOptions>): void;
export declare function $default(key: string): any;
export declare function $setdefault(key: string, value?: any): void;
export declare function $removedefault(key: string): void;
