"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSRangeSet = void 0;
const commons_1 = require("./commons");
const tserrors_1 = require("./tserrors");
const tslist_1 = require("./tslist");
const tsrange_1 = require("./tsrange");
const types_1 = require("./types");
class TSRangeSet extends tslist_1.TSList {
    constructor(v) {
        super();
        if ((0, commons_1.$ok)(v)) {
            if (v instanceof TSRangeSet) {
                v.forEach(r => super.add(r));
                return;
            }
            if (!(0, commons_1.$isarray)(v)) {
                if ((0, commons_1.$isnumber)(v)) {
                    v = [new tsrange_1.TSRange(v, 1)];
                }
                else if ((v instanceof tsrange_1.TSRange) || (0, tsrange_1.$comformsToInterval)(v)) {
                    v = [v];
                }
                else {
                    throw new tserrors_1.TSError('TSRangeSet.constructor(): invalid parameter', { parameter: v });
                }
            }
            let u = v;
            let tmp = [];
            if ((0, commons_1.$count)(u) === 2 && (0, commons_1.$isnumber)(u[0]) && (0, commons_1.$isnumber)(u[1])) {
                const r = tsrange_1.TSRange.fromArray(u);
                if ((0, commons_1.$ok)(r)) {
                    u = [r];
                }
                else {
                    throw new tserrors_1.TSError('TSRangeSet.constructor(): invalid range as array parameter', { parameter: v });
                }
            }
            for (let e of u) {
                let r = undefined;
                if (e instanceof tsrange_1.TSRange) {
                    r = e;
                }
                else if ((0, commons_1.$isarray)(e)) {
                    r = tsrange_1.TSRange.fromArray(e);
                }
                else if ((0, tsrange_1.$comformsToInterval)(e) && e.hasSignificantRange) {
                    r = e.range;
                }
                if (!(0, commons_1.$ok)(r)) {
                    throw new tserrors_1.TSError('TSRangeSet.constructor(): invalid array parameter', { parameter: v });
                    ;
                }
                else if (!r.hasSignificantRange) {
                    throw new tserrors_1.TSError('TSRangeSet.constructor(): invalid range parameter', { parameter: v });
                    ;
                }
                else {
                    tmp.push(r);
                }
            }
            for (let r of tmp) {
                this._addRange(r);
            }
        }
    }
    get length() {
        let n = 0;
        this.forEach(r => n += r.length);
        return n;
    }
    get hasSignificantRange() { return (0, commons_1.$ok)(super.first); }
    get range() {
        if (this.hasSignificantRange) {
            const loc = super.first.data.location;
            return new tsrange_1.TSRange(loc, super.last.data.maxRange - loc);
        }
        return (0, tsrange_1.TSBadRange)();
    }
    get isValid() { return this.range.isValid; }
    get isEmpty() { return this.range.isEmpty; }
    get location() { return (0, commons_1.$ok)(super.first) ? super.first.data.location : NaN; }
    get maxRange() { return (0, commons_1.$ok)(super.first) ? super.last.data.maxRange : NaN; }
    clone() { return new TSRangeSet(this); }
    _addRange(r) {
        let l = this.first;
        while (l !== null) {
            if (l.data.continuousWith(r)) {
                l.data = l.data.unionRange(r);
                while (l !== null && l.next !== null && l.data.continuousWith(l.next.data)) {
                    l.data = l.data.unionRange(l.next.data);
                    super.removeNode(l.next);
                }
                return;
            }
            else if (r.maxRange < l.data.location) {
                super.insert(r, l);
                return;
            }
            l = l.next;
        }
        super.add(r);
    }
    _removeRange(r) {
        if (this.intersects(r)) {
            let l = this.first;
            while (l !== null && r.maxRange >= l.data.location) {
                const inter = r.intersectionRange(l.data);
                if (l.data.isEqual(inter)) {
                    let n = l.next;
                    super.removeNode(l);
                    l = n;
                }
                else {
                    if (!inter.isEmpty) {
                        if (inter.location > l.data.location) {
                            if (inter.maxRange < l.data.maxRange) {
                                const upRange = new tsrange_1.TSRange(inter.maxRange, l.data.maxRange - inter.maxRange);
                                if (l.next)
                                    super.insert(upRange, l.next);
                                else
                                    super.add(upRange);
                            }
                            l.data.length = inter.location - l.data.location;
                        }
                        else if (inter.location == l.data.location) {
                            l.data = new tsrange_1.TSRange(inter.maxRange, l.data.maxRange - inter.maxRange);
                        }
                    }
                    l = l.next;
                }
            }
        }
    }
    _intersectRange(r) {
        if (this.hasSignificantRange) {
            if (this.range.intersects(r)) {
                let l = this.first;
                while (l && !l.data.intersects(r)) {
                    let n = l.next;
                    super.removeNode(l);
                    l = n;
                }
                let inter;
                while (l && (inter = r.intersectionRange(l.data)).length > 0) {
                    l.data = inter;
                    l = l.next;
                }
                while (l) {
                    let n = l.next;
                    super.removeNode(l);
                    l = n;
                }
            }
            else {
                this.clear();
            }
        }
    }
    contains(v) {
        if (!this.hasSignificantRange)
            return false;
        if (typeof v === 'number') {
            return this.contains(new tsrange_1.TSRange(v, 1));
        }
        else if ((0, commons_1.$isarray)(v)) {
            const rs = tsrange_1.TSRange.fromArray(v);
            if (!(0, commons_1.$ok)(rs) || !(rs === null || rs === void 0 ? void 0 : rs.hasSignificantRange)) {
                throw new tserrors_1.TSError('TSRangeSet.contains(): invalid range array parameter', { parameter: v });
            }
            return (0, commons_1.$ok)(this.search(r => r.contains(rs)));
        }
        else if (v instanceof tsrange_1.TSRange) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.contains(): invalid range parameter', { parameter: v });
            }
            return (0, commons_1.$ok)(this.search(r => r.contains(v)));
        }
        else if (v instanceof TSRangeSet) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.contains(): invalid range set parameter', { parameter: v });
            }
            return this.range.contains(v.range) && !(0, commons_1.$ok)(v.search(r => !this.contains(r)));
        }
        if (!(0, tsrange_1.$comformsToInterval)(v) || !v.hasSignificantRange) {
            throw new tserrors_1.TSError('TSRangeSet.contains(): invalid interval parameter', { parameter: v });
        }
        return this.contains(v.range);
    }
    intersects(v) {
        if (!this.hasSignificantRange)
            return false;
        if (typeof v === 'number') {
            return this.intersects(new tsrange_1.TSRange(v, 1));
        }
        else if ((0, commons_1.$isarray)(v)) {
            const rs = tsrange_1.TSRange.fromArray(v);
            if (!(0, commons_1.$ok)(rs) || !(rs === null || rs === void 0 ? void 0 : rs.hasSignificantRange)) {
                throw new tserrors_1.TSError('TSRangeSet.intersects(): invalid range array parameter', { parameter: v });
            }
            return (0, commons_1.$ok)(this.search(r => r.intersects(rs)));
        }
        else if (v instanceof tsrange_1.TSRange) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.intersects(): invalid range parameter', { parameter: v });
            }
            return (0, commons_1.$ok)(this.search(r => r.intersects(v)));
        }
        else if (v instanceof TSRangeSet) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.intersects(): invalid range set parameter', { parameter: v });
            }
            return this.range.intersects(v.range) && !(0, commons_1.$ok)(v.search(r => !this.intersects(r)));
        }
        if (!(0, tsrange_1.$comformsToInterval)(v) || !v.hasSignificantRange) {
            throw new tserrors_1.TSError('TSRangeSet.intersects(): invalid interval parameter', { parameter: v });
        }
        return this.intersects(v.range);
    }
    unionWidth(v) {
        if (typeof v === 'number') {
            this._addRange(new tsrange_1.TSRange(v, 1));
        }
        else if ((0, commons_1.$isarray)(v)) {
            const r = tsrange_1.TSRange.fromArray(v);
            if (!(0, commons_1.$ok)(r) || !(r === null || r === void 0 ? void 0 : r.hasSignificantRange)) {
                throw new tserrors_1.TSError('TSRangeSet.unionWidth(): invalid range array parameter', { parameter: v });
            }
            this._addRange(r);
        }
        else if (v instanceof tsrange_1.TSRange) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.unionWidth(): invalid range parameter', { parameter: v });
            }
            this._addRange(v);
        }
        else if (v instanceof TSRangeSet) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.unionWidth(): invalid range set parameter', { parameter: v });
            }
            v.forEach(r => this._addRange(r));
        }
        else {
            if (!(0, tsrange_1.$comformsToInterval)(v) || !v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.unionWidth(): invalid interval parameter', { parameter: v });
            }
            this._addRange(v.range);
        }
    }
    substractFrom(v) {
        if (typeof v === 'number') {
            this._removeRange(new tsrange_1.TSRange(v, 1));
        }
        else if ((0, commons_1.$isarray)(v)) {
            const r = tsrange_1.TSRange.fromArray(v);
            if (!(0, commons_1.$ok)(r) || !(r === null || r === void 0 ? void 0 : r.hasSignificantRange)) {
                throw new tserrors_1.TSError('TSRangeSet.substractFrom(): invalid range array parameter', { parameter: v });
            }
            this._removeRange(r);
        }
        else if (v instanceof tsrange_1.TSRange) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.substractFrom(): invalid range parameter', { parameter: v });
            }
            this._removeRange(v);
        }
        else if (v instanceof TSRangeSet) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.substractFrom(): invalid range set parameter', { parameter: v });
            }
            v.forEach(r => this._removeRange(r));
        }
        else {
            if (!(0, tsrange_1.$comformsToInterval)(v) || !v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.substractFrom(): invalid interval parameter', { parameter: v });
            }
            this._removeRange(v.range);
        }
    }
    intersectWidth(v) {
        if (typeof v === 'number') {
            this._intersectRange(new tsrange_1.TSRange(v, 1));
        }
        else if ((0, commons_1.$isarray)(v)) {
            const r = tsrange_1.TSRange.fromArray(v);
            if (!(0, commons_1.$ok)(r) || !(r === null || r === void 0 ? void 0 : r.hasSignificantRange)) {
                throw new tserrors_1.TSError('TSRangeSet.intersectWidth(): invalid range array parameter', { parameter: v });
            }
            this._intersectRange(r);
        }
        else if (v instanceof tsrange_1.TSRange) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.intersectWidth(): invalid range parameter', { parameter: v });
            }
            this._intersectRange(v);
        }
        else if (v instanceof TSRangeSet) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.intersectWidth(): invalid range set parameter', { parameter: v });
            }
            v.forEach(r => this._intersectRange(r));
        }
        else {
            if (!(0, tsrange_1.$comformsToInterval)(v) || !v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.intersectWidth(): invalid interval parameter', { parameter: v });
            }
            this._intersectRange(v.range);
        }
    }
    union(v) {
        let ret = new TSRangeSet(this);
        ret.unionWidth(v);
        return ret;
    }
    intersection(v) {
        let ret = new TSRangeSet(this);
        ret.intersectWidth(v);
        return ret;
    }
    substraction(v) {
        let ret = new TSRangeSet(this);
        ret.substractFrom(v);
        return ret;
    }
    complement(v) {
        if (!this.hasSignificantRange) {
            if (!(0, commons_1.$ok)(v)) {
                throw new tserrors_1.TSError('TSRangeSet.complement(): call with no parameter on an empty range set', { parameter: v });
            }
            return new TSRangeSet(v);
        }
        if (!(0, commons_1.$ok)(v)) {
            v = this.range;
        }
        else if ((0, commons_1.$isarray)(v)) {
            const r = tsrange_1.TSRange.fromArray(v);
            if (!(0, commons_1.$ok)(r) || !(r === null || r === void 0 ? void 0 : r.hasSignificantRange)) {
                throw new tserrors_1.TSError('TSRangeSet.complement(): invalid range array parameter', { parameter: v });
            }
            v = r;
        }
        else if (v instanceof tsrange_1.TSRange) {
            if (!v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.complement(): invalid range parameter', { parameter: v });
            }
        }
        else {
            if (!(0, tsrange_1.$comformsToInterval)(v) || !v.hasSignificantRange) {
                throw new tserrors_1.TSError('TSRangeSet.complement(): invalid interval parameter', { parameter: v });
            }
            v = v.range;
        }
        let ret = new TSRangeSet(v);
        ret.substractFrom(this);
        return ret;
    }
    isEqual(other) {
        return this === other || (other instanceof TSRangeSet && super.isEqual(other));
    }
    compare(other) {
        if (this.isEqual(other)) {
            return types_1.Same;
        }
        if ((other instanceof tsrange_1.TSRange || other instanceof TSRangeSet) && this.hasSignificantRange && other.hasSignificantRange) {
            if (this.location >= other.maxRange) {
                return types_1.Descending;
            }
            if (other.location >= this.maxRange) {
                return types_1.Ascending;
            }
        }
        return undefined;
    }
}
exports.TSRangeSet = TSRangeSet;
//# sourceMappingURL=tsrangeset.js.map